package data.hullmods;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.*;
import com.fs.starfarer.api.ui.Alignment;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;

import java.awt.*;

public class CbpsMissileFabricator extends BaseHullMod {
    private static final String[] ALL_INCOMPAT_IDS = new String[]{
            "missleracks", "missile_autoloader", "missile_reload", "eis_microforge", "drg_missileforge", "khs_missile_reloader", "hhe_MissileReload", "MHMods_prefabricator", "tahlan_vundautoforge", "tahlan_scalarautoforge", "sst_cnf_mis",
            "sst_pnf_mis", "bbplus_missile_fabricator", "specialsphmod_pristinenanoforge_upgrades", "specialsphmod_corruptednanoforge_upgrades"};

    public static String MA_DATA_KEY = "core_reloader_data_key";
    protected static Object STATUSKEY = new Object();

    public void applyEffectsBeforeShipCreation(ShipAPI.HullSize hullSize, MutableShipStatsAPI stats, String id) {
        stats.getMissileRoFMult().modifyMult(id, 1.11F);
        stats.getMissileAmmoBonus().modifyPercent(id, 20.0F);
    }

    public void applyEffectsAfterShipCreation(ShipAPI ship, String id) {
        for (String tmp : ALL_INCOMPAT_IDS) {
            if (ship.getVariant().getHullMods().contains(tmp)) {
                ship.getVariant().removeMod(tmp);
                CbpsHullmodDisplayBlockScript.showBlocked(ship);
            }
        }
    }

    public void advanceInCombat(ShipAPI ship, float amount) {
        CombatEngineAPI engine = Global.getCombatEngine();
        String key = MA_DATA_KEY + "_" + ship.getId();
        if (engine.isPaused() || !ship.isAlive())
            return;
        MissileAutoforger data = (MissileAutoforger)engine.getCustomData().get(key);
        if (data == null) {
            data = new MissileAutoforger();
            engine.getCustomData().put(key, data);
        }
        boolean sneeed = false;
        for (WeaponAPI w : ship.getAllWeapons()) {
            if (w.getType() != WeaponAPI.WeaponType.MISSILE)
                continue;
            if (w.usesAmmo() && w.getAmmo() < w.getMaxAmmo())
                sneeed = true;
        }
        if (sneeed) {
            data.interval.advance(amount);
            int elapsed = Math.round((int)data.interval.getElapsed());
            if (data.interval.intervalElapsed()) {
                for (WeaponAPI w : ship.getAllWeapons()) {
                    if (w.getType() != WeaponAPI.WeaponType.MISSILE)
                        continue;
                    int currentAmmo = w.getAmmo();
                    int maxAmmo = w.getMaxAmmo();
                    if (w.usesAmmo() && currentAmmo < maxAmmo) {
                        int numerator = (int)Math.max(1.0F, w.getSpec().getMaxAmmo() / 4.0F);
                        int reloadCount = numerator;
                        int newAmmo = currentAmmo + reloadCount;
                        if (newAmmo + numerator >= maxAmmo) {
                            reloadCount = maxAmmo - currentAmmo;
                            w.setAmmo(maxAmmo);
                        } else {
                            w.setAmmo(newAmmo);
                        }
                        engine.addFloatingText(w.getLocation(), "+" + reloadCount, 25.0F, Color.GREEN, (CombatEntityAPI)ship, 0.0F, 0.0F);
                    }
                    Global.getSoundPlayer().playSound("system_forgevats", 1.0F, 1.0F, ship.getLocation(), ship.getVelocity());
                }
            } else if (ship == Global.getCombatEngine().getPlayerShip()) {
                Global.getCombatEngine().maintainStatusForPlayerShip(STATUSKEY, Global.getSettings().getSpriteName("tooltips", "nanoforge_missile_fab"), "Status: Forging Missiles", elapsed + " / 50(sec) until reload", false);
            }
        }
    }

    public boolean isApplicableToShip(ShipAPI ship) {
        return (ship != null &&
                !ship.getVariant().getHullMods().contains("missleracks") &&
                !ship.getVariant().getHullMods().contains("missile_autoloader") &&
                !ship.getVariant().getHullMods().contains("missile_reload") &&
                !ship.getVariant().getHullMods().contains("drg_missileforge") &&
                !ship.getVariant().getHullMods().contains("eis_microforge") &&
                !ship.getVariant().getHullMods().contains("khs_missile_reloader") &&
                !ship.getVariant().getHullMods().contains("hhe_MissileReload") &&
                !ship.getVariant().getHullMods().contains("MHMods_prefabricator") &&
                !ship.getVariant().getHullMods().contains("tahlan_vundautoforge") &&
                !ship.getVariant().getHullMods().contains("tahlan_scalarautoforge") &&
                !ship.getVariant().getHullMods().contains("sst_cnf_mis") &&
                !ship.getVariant().getHullMods().contains("sst_pnf_mis") &&
                !ship.getVariant().getHullMods().contains("bbplus_missile_fabricator") &&
                !ship.getVariant().getHullMods().contains("specialsphmod_corruptednanoforge_upgrades") &&
                !ship.getVariant().getHullMods().contains("specialsphmod_pristinenanoforge_upgrades"));
    }

    public String getUnapplicableReason(ShipAPI ship) {
        if (ship.getVariant().getHullMods().contains("missleracks"))
            return "Incompatible with Expanded Missile Racks";
        if (ship.getVariant().getHullMods().contains("missile_autoloader"))
            return "Incompatible with Missile Autoloader";
        if (ship.getVariant().getHullMods().contains("missile_reload"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("drg_missileforge"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("eis_microforge"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("khs_missile_reloader"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("MHMods_prefabricator"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("hhe_MissileReload"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("tahlan_vundautoforge"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("tahlan_scalarautoforge"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("sst_cnf_mis"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("sst_pnf_mis"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("bbplus_missile_fabricator"))
            return "Incompatible with this autoforge";
        if (ship.getVariant().getHullMods().contains("specialsphmod_corruptednanoforge_upgrades"))
            return "Incompatible with this autoforge";
        return ship.getVariant().getHullMods().contains("specialsphmod_pristinenanoforge_upgrades") ? "Incompatible with this autoforge" : null;
    }

    public String getDescriptionParam(int index, ShipAPI.HullSize hullSize) {
        return null;
    }

    public void addPostDescriptionSection(TooltipMakerAPI tooltip, ShipAPI.HullSize hullSize, ShipAPI ship, float width, boolean isForModSpec) {
        Color green = new Color(55, 245, 65, 255);
        float pad = 10.0F;
        float padNote = 5.0F;
        Color bad = Misc.getNegativeHighlightColor();
        Color badbg = new Color(128, 38, 0, 175);
        tooltip.addSectionHeading("Details", Alignment.MID, 10.0F);
        tooltip.addPara("- Improved missile reload rate: %s\n- Increases ammo capacity of missile weapons: %s\n- Replenish all missile weapons by %s of max ammo(min of %s) for every %s seconds.", 10.0F, green, new String[] { Misc.getRoundedValue(10.0F) + "%",
                Misc.getRoundedValue(20.0F) + "%", "one-fourth",
                Misc.getRoundedValue(1.0F), Misc.getRoundedValue(50.0F) });
        tooltip.addSectionHeading("Incompatibilities", bad, badbg, Alignment.MID, 10.0F);
        tooltip.addPara("Incompatible with %s, %s", 0.0F, Misc.getNegativeHighlightColor(), new String[] { "Expanded Missile Racks", "Missile Autoloader" });
    }

    public static class MissileAutoforger {
        final IntervalUtil interval = new IntervalUtil(50.0F, 50.0F);
    }
}
