package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.GenericPluginManagerAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.combat.ShipHullSpecAPI;
import com.fs.starfarer.api.combat.ShipVariantAPI;
import com.fs.starfarer.api.combat.WeaponAPI;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.impl.campaign.procgen.themes.CapitalOmegaDefenderPluginImpl;
import com.fs.starfarer.api.impl.campaign.procgen.themes.PKDefenderPluginImpl;
import com.fs.starfarer.api.loading.FighterWingSpecAPI;
import com.fs.starfarer.api.loading.WeaponSpecAPI;
import com.fs.starfarer.api.util.Misc;
import data.scripts.world.CbpsGen;
import exerelin.campaign.MarketDescChanger;
import exerelin.campaign.MiscEventsManager;
import exerelin.campaign.SectorManager;
import exerelin.campaign.TransponderCheckBlockScript;
import exerelin.campaign.ai.MilitaryInfoHelper;
import exerelin.campaign.battle.EncounterLootHandler;
import exerelin.campaign.econ.EconomyInfoHelper;
import exerelin.campaign.fleets.PlayerInSystemTracker;
import exerelin.campaign.graphics.MiningCooldownDrawerV2;
import exerelin.campaign.ui.FieldOptionsScreenScript;
import exerelin.campaign.ui.PlayerFactionSetupNag;
import exerelin.utilities.NexConfig;
import exerelin.world.SSP_AsteroidTracker;
import exerelin.world.scenarios.DerelictEmpireOfficerGeneratorPlugin;

import java.util.Arrays;
import java.util.List;

public class CbpsAIBPModPlugin extends BaseModPlugin {
    //private final boolean hasHMI = Global.getSettings().getModManager().isModEnabled("HMI_brighton");
    private final boolean hasBrighton = Global.getSettings().getModManager().isModEnabled("HMI_brighton");

    @Override
    public void onNewGame() {
        SectorAPI sector = Global.getSector();

        //boolean haveNexerelin = Global.getSettings().getModManager().isModEnabled("nexerelin");
        //if (!haveNexerelin || SectorManager.getManager().isCorvusMode()) {
            new CbpsGen().generate(sector);
        //}
    }

    @Override
    public void onGameLoad(boolean newGame) {
        addTransientScriptsAndListeners(newGame);
    }

    protected void addTransientScriptsAndListeners(boolean newGame) {
        SectorAPI sector = Global.getSector();

        GenericPluginManagerAPI plugins = sector.getGenericPlugins();
        if (!plugins.hasPlugin(CapitalOmegaDefenderPluginImpl.class)) {
            plugins.addPlugin(new CapitalOmegaDefenderPluginImpl(), true);
        }
    }

    @Override
    public void onApplicationLoad() {
        /*ShipHullSpecAPI guardian = Global.getSettings().getHullSpec("guardian");
        guardian.addTag("cbps_guardian_bp");
        if (!guardian.hasTag("auto_rec")) {
            guardian.addTag("auto_rec");
        }*/

        for (String varID : Global.getSettings().getAllVariantIds()) {
            ShipVariantAPI variant = Global.getSettings().getVariant(varID);
            ShipHullSpecAPI spec = variant.getHullSpec();
            if (spec.getHints().contains(ShipHullSpecAPI.ShipTypeHints.STATION)) continue;

            if (spec.hasTag("remnant") && spec.hasTag("auto_rec")) {
                spec.addTag("cbps_remnants_bp");
            }
            if (spec.hasTag("derelict") && spec.hasTag("auto_rec")) {
                if (spec == Global.getSettings().getHullSpec("guardian")) continue; //Guardian goes into own BP
                spec.addTag("cbps_derelict_bp");
            }
            if (spec.hasTag("domres_derelict") && spec.hasTag("auto_rec") && hasBrighton) {
                spec.addTag("cbps_domres_bp");
            }
        }

        for (FighterWingSpecAPI fighter : Global.getSettings().getAllFighterWingSpecs()) {
            if (fighter.hasTag(Tags.NO_DROP)) continue;

            if (fighter.hasTag("remnant")) {
                fighter.addTag("cbps_remnants_bp");
            }

            if (fighter.hasTag("derelict")) {
                fighter.addTag("cbps_derelict_bp");
            }
        }

        for (WeaponSpecAPI wep : Global.getSettings().getAllWeaponSpecs()) {
            if (wep.hasTag(Tags.NO_DROP) || wep.getAIHints().contains(WeaponAPI.AIHints.SYSTEM)) continue;

            if (wep.hasTag("remnant") && wep.getManufacturer().equals("Remnant")) {
                wep.addTag("cbps_remnants_weapons_bp");
            }
            //if (wep.hasTag("omega")) {
            //    wep.addTag("cbps_omega_weapons_bp");
            //}
        }

        List<String> omegaWeaponPackage1 = Arrays.asList("minipulser", "disintegrator", "vpdriver");
        List<String> omegaWeaponPackage2 = Arrays.asList("riftlance", "riftbeam", "riftcascade");
        List<String> omegaWeaponPackage3 = Arrays.asList("cryoflux", "cryoblaster");
        List<String> omegaWeaponPackage4 = Arrays.asList("shockrepeater", "realitydisruptor");
        List<String> omegaWeaponPackage5 = Arrays.asList("amsrm", "resonatormrm", "rifttorpedo");

        for (String weapon : omegaWeaponPackage1) {
            WeaponSpecAPI weaponSpec = Global.getSettings().getWeaponSpec(weapon);
            weaponSpec.addTag("cbps_omega_weapons1_bp");
        }

        for (String weapon : omegaWeaponPackage2) {
            WeaponSpecAPI weaponSpec = Global.getSettings().getWeaponSpec(weapon);
            weaponSpec.addTag("cbps_omega_weapons2_bp");
        }

        for (String weapon : omegaWeaponPackage3) {
            WeaponSpecAPI weaponSpec = Global.getSettings().getWeaponSpec(weapon);
            weaponSpec.addTag("cbps_omega_weapons3_bp");
        }

        for (String weapon : omegaWeaponPackage4) {
            WeaponSpecAPI weaponSpec = Global.getSettings().getWeaponSpec(weapon);
            weaponSpec.addTag("cbps_omega_weapons4_bp");
        }

        for (String weapon : omegaWeaponPackage5) {
            WeaponSpecAPI weaponSpec = Global.getSettings().getWeaponSpec(weapon);
            weaponSpec.addTag("cbps_omega_weapons5_bp");
        }
    }
}

