package data.scripts.campaign.customstart;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.Script;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.CargoAPI;
import com.fs.starfarer.api.campaign.InteractionDialogAPI;
import com.fs.starfarer.api.campaign.SpecialItemData;
import com.fs.starfarer.api.campaign.rules.MemKeys;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.characters.CharacterCreationData;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.Skills;
import com.fs.starfarer.api.impl.campaign.rulecmd.AddRemoveCommodity;
import com.fs.starfarer.api.impl.campaign.rulecmd.NGCAddStandardStartingScript;
import com.fs.starfarer.api.util.Misc;
import exerelin.campaign.ExerelinSetupData;
import exerelin.campaign.PlayerFactionStore;
import exerelin.campaign.customstart.CustomStart;
import exerelin.utilities.StringHelper;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

public class CbpsAIResearcherStart extends CustomStart {

    private boolean hasHMI = Global.getSettings().getModManager().isModEnabled("HMI_brighton");

    @Override
    public void execute(InteractionDialogAPI dialog, Map<String, MemoryAPI> memoryMap) {
        CharacterCreationData data = (CharacterCreationData) memoryMap.get(MemKeys.LOCAL).get("$characterData");

        PlayerFactionStore.setPlayerFactionIdNGC(Factions.TRITACHYON);

        CampaignFleetAPI tempFleet = FleetFactoryV3.createEmptyFleet(
                PlayerFactionStore.getPlayerFactionIdNGC(), FleetTypes.PATROL_SMALL, null);

        addFleetMember("cbps_brilliant_refit_Standard", dialog, data, tempFleet, "flagship");
        addFleetMember("scintilla_Strike", dialog, data, tempFleet, "none");
        addFleetMember("berserker_Assault", dialog, data, tempFleet, "none");
        addFleetMember("lumen_Standard", dialog, data, tempFleet, "none");
        addFleetMember("lumen_Standard", dialog, data, tempFleet, "none");
        addFleetMember("glimmer_Assault", dialog, data, tempFleet, "none");
        addFleetMember("buffalo_tritachyon_Standard", dialog, data, tempFleet, "none");
        addFleetMember("dram_Light", dialog, data, tempFleet, "none");

        // can't do it like this, if you back out then pick another start, you'll still have A.S.
        //MutableCharacterStatsAPI stats = data.getPerson().getStats();
        //stats.setSkillLevel(Skills.AUTOMATED_SHIPS, 1);

        dialog.getTextPanel().setFontSmallInsignia();
        dialog.getTextPanel().addParagraph("Gained skill: Automated Ships", Misc.getPositiveHighlightColor());
        dialog.getTextPanel().highlightInLastPara(Global.getSettings().getSkillSpec(Skills.AUTOMATED_SHIPS).getGoverningAptitudeColor().brighter(), "Automated Ships");
        dialog.getTextPanel().setFontInsignia();

        data.getStartingCargo().getCredits().add(15000);
        AddRemoveCommodity.addCreditsGainText(15000, dialog.getTextPanel());

        tempFleet.getFleetData().setSyncNeeded();
        tempFleet.getFleetData().syncIfNeeded();
        tempFleet.forceSync();

        int crew = 0;
        int fuel = 0;
        int supplies = 0;
        for (FleetMemberAPI member : tempFleet.getFleetData().getMembersListCopy()) {
            crew += member.getMinCrew() + (int) ((member.getMaxCrew() - member.getMinCrew()) * 0.3f);
            fuel += (int) member.getFuelCapacity() * 0.5f;
            supplies += (int) member.getBaseDeploymentCostSupplies() * 3;
        }
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, Commodities.CREW, crew);
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, Commodities.FUEL, fuel);
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, Commodities.SUPPLIES, supplies);
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, Commodities.HEAVY_MACHINERY, 40);
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, "alpha_core", 1.0F);
        data.getStartingCargo().addItems(CargoAPI.CargoItemType.RESOURCES, "beta_core", 3.0F);

        AddRemoveCommodity.addCommodityGainText(Commodities.CREW, crew, dialog.getTextPanel());
        AddRemoveCommodity.addCommodityGainText(Commodities.FUEL, fuel, dialog.getTextPanel());
        AddRemoveCommodity.addCommodityGainText(Commodities.SUPPLIES, supplies, dialog.getTextPanel());
        AddRemoveCommodity.addCommodityGainText(Commodities.HEAVY_MACHINERY, 40, dialog.getTextPanel());
        AddRemoveCommodity.addCommodityGainText("alpha_core", 1, dialog.getTextPanel());
        AddRemoveCommodity.addCommodityGainText("beta_core", 3, dialog.getTextPanel());

        data.getStartingCargo().addSpecial(new SpecialItemData("cbps_derelict_bp_package", null), 1.0F);
        data.getStartingCargo().addSpecial(new SpecialItemData("cbps_remnants_bp_package", null), 1.0F);
        if (hasHMI) {
            data.getStartingCargo().addSpecial(new SpecialItemData("cbps_domres_bp_package", null), 1.0F);
            data.getStartingCargo().addSpecial(new SpecialItemData("cbps_remnants_weapons_bp_package", null), 1.0F);
        }

        PlayerFactionStore.setPlayerFactionIdNGC(Factions.PLAYER);
        ExerelinSetupData.getInstance().freeStart = true;

        data.addScript(new Script() {
            public void run() {
                CampaignFleetAPI fleet = Global.getSector().getPlayerFleet();

                // SEQUENCE BREAK!
                Global.getSector().getPlayerPerson().getStats().setSkillLevel(Skills.AUTOMATED_SHIPS, 1);

                NGCAddStandardStartingScript.adjustStartingHulls(fleet);

                fleet.getFleetData().ensureHasFlagship();

                for (FleetMemberAPI member : fleet.getFleetData().getMembersListCopy()) {
                    float max = member.getRepairTracker().getMaxCR();
                    member.getRepairTracker().setCR(max);
                }
                fleet.getFleetData().setSyncNeeded();
            }
        });

        dialog.getVisualPanel().showFleetInfo(StringHelper.getString("exerelin_ngc", "playerFleet", true),
                tempFleet, null, null);

        dialog.getOptionPanel().addOption(StringHelper.getString("done", true), "nex_NGCDone");
        dialog.getOptionPanel().addOption(StringHelper.getString("back", true), "nex_NGCStartBack");
    }

    public void addFleetMember(String vid, InteractionDialogAPI dialog, CharacterCreationData data, CampaignFleetAPI fleet, String special) {
        data.addStartingFleetMember(vid, FleetMemberType.SHIP);
        FleetMemberAPI temp = Global.getFactory().createFleetMember(FleetMemberType.SHIP, vid);
        fleet.getFleetData().addFleetMember(temp);
        temp.getRepairTracker().setCR(0.7f);

        if (special.equals("flagship")) {
            fleet.getFleetData().setFlagship(temp);
            temp.setCaptain(data.getPerson());
        }
    }
}