package data.scripts.world.systems;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.*;
import com.fs.starfarer.api.impl.campaign.ids.*;
import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.CbpsOmegaStationFleetManager;
import com.fs.starfarer.api.impl.campaign.procgen.themes.RemnantThemeGenerator;
import com.fs.starfarer.api.util.Misc;
import org.lwjgl.util.vector.Vector2f;

import java.awt.*;

import static com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator.random;

public class CbpsOmegaBPSystem {
    public static final Vector2f LOCATION = new Vector2f(-62000, 28000);
    public static String CBPS_OMEGA_CACHE_KEY = "$cbps_OmegaCacheKey";

    public void generate(SectorAPI sector) {
        //Init
        StarSystemAPI system = sector.createStarSystem("Omegon");
        String systemName = "Omegon";    // will be used later
        system.setType(StarSystemGenerator.StarSystemType.TRINARY_2CLOSE);
        system.setAge(StarAge.YOUNG);
        system.getLocation().set(LOCATION);
        system.setLightColor(new Color(204, 0, 128));
        system.setBackgroundTextureFilename("graphics/backgrounds/cbps_background1.jpg");
        SectorEntityToken center = system.initNonStarCenter();

        PlanetAPI orbitStar1 = system.addPlanet("cbps_omega_system_orbit_star1",
                center,
                "Omegon 1",
                StarTypes.WHITE_DWARF,
                0, 400, 1200, 30);
        system.setStar(orbitStar1);
        system.addCorona(orbitStar1, 250, 2f, 0.1f, 2f);

        PlanetAPI orbitStar2 = system.addPlanet("cbps_omega_system_orbit_star2",
                center,
                "Omegon 2",
                StarTypes.WHITE_DWARF,
                120, 400, 1200, 30);
        system.setSecondary(orbitStar2);
        system.addCorona(orbitStar2, 250, 2f, 0.1f, 2f);

        PlanetAPI orbitStar3 = system.addPlanet("cbps_omega_system_orbit_star3",
                center,
                "Omegon 3",
                StarTypes.WHITE_DWARF,
                240, 400, 1200, 30);
        system.setTertiary(orbitStar3);
        system.addCorona(orbitStar3, 250, 2f, 0.1f, 2f);

        SectorEntityToken lootStation1 = RemnantThemeGenerator.addSalvageEntity(system, "cbps_omega_bp_station1", Factions.REMNANTS);
        lootStation1.setId("cbps_omega_system_loot1");
        lootStation1.setCircularOrbitPointingDown(center, 50, 1200, 30);
        lootStation1.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        SectorEntityToken lootStation2 = RemnantThemeGenerator.addSalvageEntity(system, "cbps_omega_bp_station2", Factions.REMNANTS);
        lootStation2.setId("cbps_omega_system_loot2");
        lootStation2.setCircularOrbitPointingDown(center, 70, 1200, 30);
        lootStation2.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        SectorEntityToken lootStation3 = RemnantThemeGenerator.addSalvageEntity(system, "cbps_omega_bp_station3", Factions.REMNANTS);
        lootStation3.setId("cbps_omega_system_loot3");
        lootStation3.setCircularOrbitPointingDown(center, 170, 1200, 30);
        lootStation3.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        SectorEntityToken lootStation4 = RemnantThemeGenerator.addSalvageEntity(system, "cbps_omega_bp_station4", Factions.REMNANTS);
        lootStation4.setId("cbps_omega_system_loot4");
        lootStation4.setCircularOrbitPointingDown(center, 190, 1200, 30);
        lootStation4.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        SectorEntityToken lootStation5 = RemnantThemeGenerator.addSalvageEntity(system, "cbps_omega_bp_station5", Factions.REMNANTS);
        lootStation5.setId("cbps_omega_system_loot5");
        lootStation5.setCircularOrbitPointingDown(center, 300, 1200, 30);
        lootStation5.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        SectorEntityToken omegaBigSpawner = system.addCustomEntity("cbps_omega_big_farm1",null, "cbps_omega_spawner", Factions.REMNANTS);
        omegaBigSpawner.setFixedLocation(0,0);

        SectorEntityToken omegaSpawner1 = system.addCustomEntity("cbps_omega_farm1",null, "cbps_omega_spawner", Factions.REMNANTS);
        omegaSpawner1.setCircularOrbit(center, 60, 3150, 90);

        SectorEntityToken omegaSpawner2 = system.addCustomEntity("cbps_omega_farm2",null, "cbps_omega_spawner", Factions.REMNANTS);
        omegaSpawner2.setCircularOrbit(center, 180, 3150, 90);

        SectorEntityToken omegaSpawner3 = system.addCustomEntity("cbps_omega_farm3",null, "cbps_omega_spawner", Factions.REMNANTS);
        omegaSpawner3.setCircularOrbit(center, 300, 3150, 90);

        int maxFleets = 1 + random.nextInt(2);

        CbpsOmegaStationFleetManager omegaFleets1 = new CbpsOmegaStationFleetManager(
                omegaBigSpawner, 1.0f, 1, maxFleets, 10.0f, 8, 24);
        system.addScript(omegaFleets1);

        CbpsOmegaStationFleetManager omegaFleets2 = new CbpsOmegaStationFleetManager(
                omegaSpawner1, 1.0f, 1, maxFleets, 10.0f, 6, 12);
        system.addScript(omegaFleets2);

        CbpsOmegaStationFleetManager omegaFleets3 = new CbpsOmegaStationFleetManager(
                omegaSpawner2, 1.0f, 1, maxFleets, 10.0f, 6, 12);
        system.addScript(omegaFleets3);

        CbpsOmegaStationFleetManager omegaFleets4 = new CbpsOmegaStationFleetManager(
                omegaSpawner3, 1.0f, 1, maxFleets, 10.0f, 6, 12);
        system.addScript(omegaFleets4);

        system.addAsteroidBelt(center, 250, 2800, 250, 30, 40, Terrain.ASTEROID_BELT, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 2700, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 2800, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 2900, 345f, null, null);

        system.addAsteroidBelt(center, 250, 3500, 250, 30, 40, Terrain.ASTEROID_BELT, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 3400, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 3500, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 3600, 345f, null, null);

        system.addAsteroidBelt(center, 250, 4200, 250, 30, 40, Terrain.ASTEROID_BELT, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 4100, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 4200, 345f, null, null);
        system.addRingBand(center, "misc", "rings_asteroids0", 256f, 3, Color.white, 256f, 4300, 345f, null, null);

        system.autogenerateHyperspaceJumpPoints(false, true);

        SectorEntityToken anchor = system.getHyperspaceAnchor();
        CustomCampaignEntityAPI beacon = Global.getSector().getHyperspace().addCustomEntity("cbps_omega_system_beacon", null, Entities.WARNING_BEACON, Factions.NEUTRAL);
        beacon.addTag(Tags.BEACON_HIGH);
        beacon.setCircularOrbitPointingDown(anchor, 100, 300, 65f);
        Color glowColor2 = new Color(125, 125, 125, 255);
        Color pingColor2 = new Color(250, 250, 250, 255);
        Misc.setWarningBeaconColors(beacon, glowColor2, pingColor2);

        SectorEntityToken gate = system.addCustomEntity("cbps_omega_system_gate", // unique id
                "Omegon Gates", // name - if null, defaultName from custom_entities.json will be used
                "inactive_gate", // type of object, defined in custom_entities.json
                null); // faction
        gate.setCircularOrbit(center, 150, 6200, 365);

        // do it last to ensure correct naming of the system
        system.setBaseName(systemName);
    }
}
