package data.scripts.world.systems;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.*;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.campaign.DerelictShipEntityPlugin;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.ids.*;
import com.fs.starfarer.api.impl.campaign.procgen.StarAge;
import com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.BaseThemeGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.CbpsOmegaStationFleetManager;
import com.fs.starfarer.api.impl.campaign.procgen.themes.RemnantThemeGenerator;
import com.fs.starfarer.api.impl.campaign.procgen.themes.SalvageSpecialAssigner;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.special.ShipRecoverySpecial;
import com.fs.starfarer.api.impl.campaign.terrain.StarCoronaTerrainPlugin;
import com.fs.starfarer.api.util.Misc;
import org.lwjgl.util.vector.Vector2f;

import java.awt.*;

import static com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator.random;
import static com.fs.starfarer.api.impl.campaign.procgen.themes.RemnantThemeGenerator.addRemnantStationInteractionConfig;

public class CbpsRemnantBPSystem {

    public static final Vector2f LOCATION = new Vector2f(-62000, -8000);

    public void generate(SectorAPI sector) {
        //Init
        StarSystemAPI system = sector.createStarSystem("Central Nexus");
        String systemName = "Central Nexus";    // will be used later
        system.setType(StarSystemGenerator.StarSystemType.NEBULA);
        system.setAge(StarAge.AVERAGE);
        system.getLocation().set(LOCATION);
        system.setLightColor(new Color(204, 0, 128));

        system.setBackgroundTextureFilename("graphics/backgrounds/cbps_background1.jpg");

        // temporarily create a "star" (needed for some things)
        PlanetAPI star = system.initStar("cbps_remnant_system_center", "nebula_center_young", 0, 0);
        //star.setSkipForJumpPointAutoGen(true);
        star.addTag(Tags.AMBIENT_LS);
        StarSystemGenerator.addSystemwideNebula(system, StarAge.AVERAGE);

        // jump point
        JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("cbps_remnant_system_jump",
                "Central Nexus Jump Point");
        jumpPoint.setStandardWormholeToHyperspaceVisual();
        system.addEntity(jumpPoint);

        // remove the star
        system.removeEntity(star);
        StarCoronaTerrainPlugin coronaPlugin = Misc.getCoronaFor(star);
        if (coronaPlugin != null) {
            system.removeEntity(coronaPlugin.getEntity());
        }
        system.setStar(null);
        SectorEntityToken center = system.initNonStarCenter();
        center.addTag(Tags.AMBIENT_LS);
        jumpPoint.setCircularOrbit(center, 60, 5250f, 361f);

        system.setStar(star);

        // Content
        /*SectorEntityToken neutralStation = system.addCustomEntity("cbps_abandoned_station_01", "Abandoned Station", "station_side05", "neutral");
        neutralStation.setCircularOrbit(center, 180, 525, orbitPeriod);
        neutralStation.setCustomDescriptionId("cbps_abandoned_station_01");
        neutralStation.setInteractionImage("illustrations", "abandoned_station3");
        Misc.setAbandonedStationMarket("cbps_abandoned_station_01", neutralStation);*/

        SectorEntityToken mainLoot = RemnantThemeGenerator.addSalvageEntity(system, "cbps_remnant_bp_station", Factions.REMNANTS);
        mainLoot.setId("cbps_remnant_system_main_loot");
        mainLoot.setCircularOrbit(center, 0, 0, 50f);

        addBattlestation(system, center, 0, 2010f, 50f);
        addBattlestation(system, center, 60f, 2010f, 50f);
        addBattlestation(system, center, 120f, 2010f, 50f);
        addBattlestation(system, center, 180f, 2010f, 50f);
        addBattlestation(system, center, 240f, 2010f, 50f);
        addBattlestation(system, center, 300f, 2010f, 50f);

        SectorEntityToken stableLoc1 = system.addCustomEntity(null, null, "stable_location", Factions.NEUTRAL);
        stableLoc1.setCircularOrbitPointingDown(star, 40f, 7250f, 361f);

        SectorEntityToken stableLoc2 = system.addCustomEntity(null, null, "stable_location", Factions.NEUTRAL);
        stableLoc2.setCircularOrbitPointingDown(star, 160f, 4550f, 361f);

        SectorEntityToken stableLoc3 = system.addCustomEntity(null, null, "stable_location", Factions.NEUTRAL);
        stableLoc3.setCircularOrbitPointingDown(star, 280f, 7250f, 361f);

        addDerelict(system, stableLoc1, "cbps_brilliant_Standard", ShipRecoverySpecial.ShipCondition.AVERAGE, 160f, true);
        addDerelict(system, stableLoc1, "cbps_brilliant_Standard", ShipRecoverySpecial.ShipCondition.BATTERED, 160f, true);
        addDerelict(system, stableLoc1, "cbps_brilliant_refit_Standard", ShipRecoverySpecial.ShipCondition.BATTERED, 270f, true);
        addDerelict(system, stableLoc1, "cbps_radiant_refit_Standard", ShipRecoverySpecial.ShipCondition.WRECKED, 320f, true);

        SectorEntityToken guardianLoot = RemnantThemeGenerator.addSalvageEntity(system, "cbps_guardian_bp_loot", Factions.REMNANTS);
        guardianLoot.setId("cbps_remnant_system_guardian_loot");
        guardianLoot.setCircularOrbit(center, 150, 6250f, 425f);
        //guardianLoot.getMemoryWithoutUpdate().set(CBPS_OMEGA_CACHE_KEY, true);

        addDerelict(system, guardianLoot, "cbps_guardian_refit_Standard", ShipRecoverySpecial.ShipCondition.WRECKED, 320f, true);

        String[] oreConditions = {Conditions.ORE_SPARSE, Conditions.ORE_MODERATE, Conditions.ORE_ABUNDANT, Conditions.ORE_RICH};
        String[] rareOreConditions = {Conditions.RARE_ORE_SPARSE, Conditions.RARE_ORE_MODERATE, Conditions.RARE_ORE_ABUNDANT, Conditions.RARE_ORE_RICH};

        PlanetAPI planet1 = system.addPlanet("cbps_remnant_system_planet1", star, "Dead Rock", "barren", 250, 85, 6750f, 825f);
        Misc.initConditionMarket(planet1);
        planet1.getMarket().addCondition(Conditions.NO_ATMOSPHERE);
        planet1.getMarket().addCondition(Conditions.DARK);
        planet1.getMarket().addCondition(Conditions.COLD);
        planet1.getMarket().addCondition(oreConditions[random.nextInt(oreConditions.length)]);
        planet1.getMarket().addCondition(rareOreConditions[random.nextInt(rareOreConditions.length)]);
        planet1.getMarket().addCondition(Conditions.RUINS_VAST);

        PlanetAPI planet2 = system.addPlanet("cbps_remnant_system_planet2", star, "Big One", "ice_giant", 40, 550, 9000f, 825f);
        Misc.initConditionMarket(planet2);
        planet2.getMarket().addCondition(Conditions.DENSE_ATMOSPHERE);
        planet2.getMarket().addCondition(Conditions.HIGH_GRAVITY);
        planet2.getMarket().addCondition(Conditions.EXTREME_WEATHER);
        planet2.getMarket().addCondition(Conditions.VERY_COLD);
        planet2.getMarket().addCondition(Conditions.DARK);
        planet2.getMarket().addCondition(Conditions.VOLATILES_PLENTIFUL);

        String[] oreConditionsCryo = {Conditions.ORE_ABUNDANT, Conditions.ORE_RICH, Conditions.ORE_ULTRARICH};
        String[] rareOreConditionsCryo = {Conditions.RARE_ORE_ABUNDANT, Conditions.RARE_ORE_RICH, Conditions.RARE_ORE_ULTRARICH};
        String[] volatilesConditionsCryo = {Conditions.VOLATILES_DIFFUSE, Conditions.VOLATILES_ABUNDANT, Conditions.VOLATILES_PLENTIFUL};

        PlanetAPI planet2a = system.addPlanet("cbps_remnant_system_planet2a", planet2, "Freezer", "cryovolcanic", 0, 100, 2075f, 100f);
        Misc.initConditionMarket(planet2a);
        planet2a.getMarket().addCondition(Conditions.EXTREME_WEATHER);
        planet2a.getMarket().addCondition(Conditions.DARK);
        planet2a.getMarket().addCondition(Conditions.VERY_COLD);
        planet2a.getMarket().addCondition(oreConditionsCryo[random.nextInt(oreConditionsCryo.length)]);
        planet2a.getMarket().addCondition(rareOreConditionsCryo[random.nextInt(rareOreConditionsCryo.length)]);
        planet2a.getMarket().addCondition(volatilesConditionsCryo[random.nextInt(volatilesConditionsCryo.length)]);

        PlanetAPI planet2b = system.addPlanet("cbps_remnant_system_planet2b", planet2, "Empty", "barren", 200, 100, 2850f, 200f);
        Misc.initConditionMarket(planet2b);
        planet2b.getMarket().addCondition(Conditions.NO_ATMOSPHERE);
        planet2b.getMarket().addCondition(Conditions.DARK);
        planet2b.getMarket().addCondition(Conditions.COLD);
        planet2b.getMarket().addCondition(oreConditions[random.nextInt(oreConditions.length)]);
        planet2b.getMarket().addCondition(rareOreConditions[random.nextInt(rareOreConditions.length)]);
        planet2b.getMarket().addCondition(Conditions.RUINS_EXTENSIVE);

        system.autogenerateHyperspaceJumpPoints(true, true);

        SectorEntityToken anchor = system.getHyperspaceAnchor();
        CustomCampaignEntityAPI beacon = Global.getSector().getHyperspace().addCustomEntity("cbps_remnant_system_beacon", null, Entities.WARNING_BEACON, Factions.NEUTRAL);
        beacon.addTag(Tags.BEACON_HIGH);
        beacon.setCircularOrbitPointingDown(anchor, 100, 300, 65f);
        Color glowColor2 = new Color(250,55,0,255);
        Color pingColor2 = new Color(250,125,0,255);
        Misc.setWarningBeaconColors(beacon, glowColor2, pingColor2);

        // do it last to ensure correct naming of the system
        system.setBaseName(systemName);
    }

    private void addBattlestation(StarSystemAPI system, SectorEntityToken center, float angle, float orbitRadius,
                                  float orbitPeriod) {

        // Spawn The Mess Station. Laughs maniacally as laptop users roll in fury.
        FleetMemberAPI member = Global.getFactory().createFleetMember(FleetMemberType.SHIP, "remnant_station2_Standard");
        member.getRepairTracker().setCR(member.getRepairTracker().getMaxCR());
        CampaignFleetAPI fleet = FleetFactoryV3.createEmptyFleet("remnant", "battlestation", null);

        //float radius_fleet1 = radius_star + radius_station + radius_variation * (float) Math.random();

        fleet.getFleetData().addFleetMember(member);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_MAKE_AGGRESSIVE, true);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_NO_JUMP, true);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_MAKE_ALLOW_DISENGAGE, true);
        fleet.setStationMode(Boolean.valueOf(true));
        addRemnantStationInteractionConfig(fleet);
        system.addEntity(fleet);

        fleet.clearAbilities();
        fleet.addAbility("transponder");
        fleet.getAbility("transponder").activate();
        fleet.getDetectedRangeMod().modifyFlat("gen", 1000.0F);
        fleet.setCircularOrbitWithSpin(
                center, // focus
                angle,// angle
                orbitRadius, // orbitRadius, was radius_fleet1, useful if random location in system
                orbitPeriod, // orbitDays
                7f, // minSpin
                12f // maxSpin
        );
        fleet.setAI(null);

        String coreId = Commodities.ALPHA_CORE;
        AICoreOfficerPlugin plugin = Misc.getAICoreOfficerPlugin(coreId);
        PersonAPI commander = plugin.createPerson(coreId, fleet.getFaction().getId(), random);
        fleet.setCommander(commander);
        fleet.getFlagship().setCaptain(commander);

        int maxFleets = 1 + random.nextInt(2);

        CbpsOmegaStationFleetManager activeFleets = new CbpsOmegaStationFleetManager(
                fleet, 1.0f, 0, maxFleets, 10.0f, 8, 24);
        system.addScript(activeFleets);
    }

    private void addDerelict(StarSystemAPI system, SectorEntityToken focus, String
            variantId, ShipRecoverySpecial.ShipCondition condition, float orbitRadius,
                             boolean recoverable) {
        DerelictShipEntityPlugin.DerelictShipData params = new DerelictShipEntityPlugin.DerelictShipData(new ShipRecoverySpecial.PerShipData(variantId, condition), false);
        SectorEntityToken ship = BaseThemeGenerator.addSalvageEntity(system, Entities.WRECK, Factions.NEUTRAL, params);
        ship.setDiscoverable(true);

        float orbitDays = orbitRadius / (10f + (float) Math.random() * 5f);
        ship.setCircularOrbit(focus, (float) Math.random() * 360f, orbitRadius, orbitDays);

        if (recoverable) {
            SalvageSpecialAssigner.ShipRecoverySpecialCreator creator = new SalvageSpecialAssigner.ShipRecoverySpecialCreator(null, 0, 0, false, null, null);
            Misc.setSalvageSpecial(ship, creator.createSpecial(ship, null));
        }
    }
}